// N-Queen Problem Hopfield-Tank Neural Network
// Written by David J. Stein, Esq. See "License.html" for terms and conditions of use.
//
// This module contains the front end for the N-Queen Problem.

#region Using declarations

using System;
using System.Threading;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

#endregion

namespace Hopfield_Tank_Neural_Network_Namespace {

	public class NQueenProblem : System.Windows.Forms.Form {

		#region Variables, initialization code, and Main

		private System.Windows.Forms.Button btnSolve;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.PictureBox picChessboard;
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.NumericUpDown nudSize;
		private System.Windows.Forms.ListView lstValues;
		private System.Windows.Forms.Button btnClear;
		private System.Windows.Forms.Button btnSolveFull;
		private System.Windows.Forms.TextBox txtStatus;
		private System.Timers.Timer timerUpdate;
		private System.Windows.Forms.Button btnReset;
		private HopfieldTankNetwork network;
		private Thread solvethread;
		private bool redraw = true;

		public NQueenProblem() {
			InitializeComponent();
			network = new HopfieldTankNetwork(8);
			solvethread = null;
		}

		[STAThread]
		static void Main() {
			Application.Run(new NQueenProblem());
		}

		#endregion

		#region Uninteresting Windows Form Designer generated code

		protected override void Dispose(bool disposing) {
			if (disposing) {
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose(disposing);
		}

		private void InitializeComponent() {
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(NQueenProblem));
			this.picChessboard = new System.Windows.Forms.PictureBox();
			this.btnSolve = new System.Windows.Forms.Button();
			this.nudSize = new System.Windows.Forms.NumericUpDown();
			this.label1 = new System.Windows.Forms.Label();
			this.lstValues = new System.Windows.Forms.ListView();
			this.btnClear = new System.Windows.Forms.Button();
			this.btnSolveFull = new System.Windows.Forms.Button();
			this.txtStatus = new System.Windows.Forms.TextBox();
			this.timerUpdate = new System.Timers.Timer();
			this.btnReset = new System.Windows.Forms.Button();
			((System.ComponentModel.ISupportInitialize)(this.nudSize)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.timerUpdate)).BeginInit();
			this.SuspendLayout();
			// 
			// picChessboard
			// 
			this.picChessboard.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.picChessboard.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.picChessboard.Location = new System.Drawing.Point(288, 8);
			this.picChessboard.Name = "picChessboard";
			this.picChessboard.Size = new System.Drawing.Size(272, 272);
			this.picChessboard.TabIndex = 1;
			this.picChessboard.TabStop = false;
			this.picChessboard.Paint += new System.Windows.Forms.PaintEventHandler(this.picChessboard_Paint);
			// 
			// btnSolve
			// 
			this.btnSolve.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.btnSolve.Location = new System.Drawing.Point(336, 288);
			this.btnSolve.Name = "btnSolve";
			this.btnSolve.Size = new System.Drawing.Size(80, 23);
			this.btnSolve.TabIndex = 2;
			this.btnSolve.Text = "Solve (&Step)";
			this.btnSolve.Click += new System.EventHandler(this.btnSolve_Click);
			// 
			// nudSize
			// 
			this.nudSize.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
			this.nudSize.Location = new System.Drawing.Point(48, 288);
			this.nudSize.Minimum = new System.Decimal(new int[] {
																					 1,
																					 0,
																					 0,
																					 0});
			this.nudSize.Name = "nudSize";
			this.nudSize.Size = new System.Drawing.Size(48, 20);
			this.nudSize.TabIndex = 0;
			this.nudSize.Value = new System.Decimal(new int[] {
																				  8,
																				  0,
																				  0,
																				  0});
			this.nudSize.ValueChanged += new System.EventHandler(this.nudSize_ValueChanged);
			// 
			// label1
			// 
			this.label1.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
			this.label1.Location = new System.Drawing.Point(8, 288);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(40, 23);
			this.label1.TabIndex = 4;
			this.label1.Text = "Size:";
			this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// lstValues
			// 
			this.lstValues.Font = new System.Drawing.Font("Microsoft Sans Serif", 6.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.lstValues.HeaderStyle = System.Windows.Forms.ColumnHeaderStyle.None;
			this.lstValues.Location = new System.Drawing.Point(8, 8);
			this.lstValues.Name = "lstValues";
			this.lstValues.Size = new System.Drawing.Size(272, 272);
			this.lstValues.TabIndex = 5;
			this.lstValues.View = System.Windows.Forms.View.Details;
			// 
			// btnClear
			// 
			this.btnClear.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.btnClear.Location = new System.Drawing.Point(280, 288);
			this.btnClear.Name = "btnClear";
			this.btnClear.Size = new System.Drawing.Size(48, 23);
			this.btnClear.TabIndex = 1;
			this.btnClear.Text = "&Clear";
			this.btnClear.Click += new System.EventHandler(this.btnClear_Click);
			// 
			// btnSolveFull
			// 
			this.btnSolveFull.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.btnSolveFull.Location = new System.Drawing.Point(424, 288);
			this.btnSolveFull.Name = "btnSolveFull";
			this.btnSolveFull.Size = new System.Drawing.Size(80, 23);
			this.btnSolveFull.TabIndex = 3;
			this.btnSolveFull.Text = "Solve (&Full)";
			this.btnSolveFull.Click += new System.EventHandler(this.btnSolveFull_Click);
			// 
			// txtStatus
			// 
			this.txtStatus.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.txtStatus.Location = new System.Drawing.Point(104, 288);
			this.txtStatus.Name = "txtStatus";
			this.txtStatus.ReadOnly = true;
			this.txtStatus.Size = new System.Drawing.Size(168, 20);
			this.txtStatus.TabIndex = 8;
			this.txtStatus.Text = "Status: Sample network loaded.";
			// 
			// timerUpdate
			// 
			this.timerUpdate.Enabled = true;
			this.timerUpdate.SynchronizingObject = this;
			this.timerUpdate.Elapsed += new System.Timers.ElapsedEventHandler(this.timerUpdate_Elapsed);
			// 
			// btnReset
			// 
			this.btnReset.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.btnReset.Location = new System.Drawing.Point(512, 288);
			this.btnReset.Name = "btnReset";
			this.btnReset.Size = new System.Drawing.Size(48, 23);
			this.btnReset.TabIndex = 4;
			this.btnReset.Text = "&Reset";
			this.btnReset.Click += new System.EventHandler(this.btnReset_Click);
			// 
			// NQueenProblem
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(568, 323);
			this.Controls.Add(this.btnReset);
			this.Controls.Add(this.txtStatus);
			this.Controls.Add(this.btnSolveFull);
			this.Controls.Add(this.btnClear);
			this.Controls.Add(this.lstValues);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.nudSize);
			this.Controls.Add(this.btnSolve);
			this.Controls.Add(this.picChessboard);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.MinimumSize = new System.Drawing.Size(576, 352);
			this.Name = "NQueenProblem";
			this.Text = "n-Queen Problem Via Hopfield-Tank Network";
			this.Resize += new System.EventHandler(this.NQueenProblem_Resize);
			this.Closing += new System.ComponentModel.CancelEventHandler(this.NQueenProblem_Closing);
			((System.ComponentModel.ISupportInitialize)(this.nudSize)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.timerUpdate)).EndInit();
			this.ResumeLayout(false);

		}

		#endregion

		#region Event handler functions

		private void nudSize_ValueChanged(object sender, System.EventArgs e) {
			network = new HopfieldTankNetwork((int) nudSize.Value);
			SetNetworkList();
			picChessboard.Refresh();
		}

		private void btnClear_Click(object sender, System.EventArgs e) {
			bool solving = (solvethread != null);
			if (solving == true)  // abort current thread
				solvethread.Suspend();
			network.Initialize();
			SetNetworkList();
			picChessboard.Refresh();
			txtStatus.Text = "Status: Ready.";
			txtStatus.Refresh();
			if (solving == true)  // start new thread
				solvethread.Resume();
		}

		private void btnSolveFull_Click(object sender, System.EventArgs e) {
			if (solvethread == null) {  // create thread and turn off buttons
				btnSolveFull.Text = "&Abort";
				btnSolveFull.Refresh();
				EnableButtons(false);
				solvethread = new Thread(new ThreadStart(Solve));
				solvethread.Start();
			}
			else {  // abort thread and turn on buttons
				solvethread.Abort();
				solvethread = null;
				btnSolveFull.Text = "Solve (&Full)";
				btnSolveFull.Refresh();
				txtStatus.Text = "Status: Aborted. E=" + network.E;
				txtStatus.Refresh();
				EnableButtons(true);
			}
		}

		private void btnSolve_Click(object sender, System.EventArgs e) {
			network.Analyze();
			SetNetworkList();
			picChessboard.Refresh();
			txtStatus.Text = "Status: E=" + network.E;
			txtStatus.Refresh();
		}

		private void picChessboard_Paint(object sender, System.Windows.Forms.PaintEventArgs e) {
			// setup drawing objects
			Font font = new Font("Times New Roman", 7.0f);
			StringFormat format = new StringFormat();
			format.Alignment = StringAlignment.Center;
			Graphics g = e.Graphics;
			g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
			g.FillRectangle(System.Drawing.Brushes.BlanchedAlmond, e.ClipRectangle);

			// draw boxes
			for (int i = 0; i < network.size; i++) {
				for (int j = 0; j < network.size; j++) {
					g.FillRectangle((i + j) % 2 == 0 ? Brushes.White : Brushes.DarkGray,
						5 + (picChessboard.Height - 14) * i / network.size,
						5 + (picChessboard.Width - 14) * j / network.size,
						(picChessboard.Width - 14) * (j + 1) / network.size - (picChessboard.Width - 14) * j / network.size,
						(picChessboard.Height - 14) * (i + 1) / network.size - (picChessboard.Height - 14) * i / network.size);
				}
			}

			// draw queens
			int edgegap = Math.Max(1, (int) ((picChessboard.Width - 14) / network.size * 0.20));
			for (int i = 0; i < network.size; i++) {
				for (int j = 0; j < network.size; j++) {
					int trans = (int) (255.0 * network.V[i,j]);
					Brush b;
					if ((network.VPrevious[i,j] > 0.995) && (network.V[i,j] > 0.995))  // assume queen is static - make it blue
						b = new SolidBrush(Color.FromArgb(0, 0, 255));					
					else if (network.VPrevious[i,j] - network.V[i,j] > 0.00000001)  // queen is disappearing - make it red
						b = new SolidBrush(Color.FromArgb(trans, 255, 0, 0));
					else if (network.VPrevious[i,j] - network.V[i,j] < -0.00000001)  // queen is appearing - make it green
						b = new SolidBrush(Color.FromArgb(trans, 0, 255, 0));
					else // queen is not fixed but is static - make it gray
						b = new SolidBrush(Color.FromArgb(trans, 128, 128, 128));
					g.FillEllipse(b, 5 + (picChessboard.Width - 14) * j / network.size + edgegap,
						5 + (picChessboard.Height - 14) * i / network.size + edgegap,
						(picChessboard.Width - 14) * (j + 1) / network.size - (picChessboard.Width - 14) * j / network.size - edgegap * 2,
						(picChessboard.Height - 14) * (i + 1) / network.size - (picChessboard.Height - 14) * i / network.size - edgegap * 2);
				}
			}
			// draw lines
			for (int i = 0; i < network.size + 1; i++) {
				g.DrawLine(Pens.Black, 5, 5 + (picChessboard.Height - 14) * i / network.size, picChessboard.Width - 9, 5 + (picChessboard.Height - 14) * i / network.size);
				g.DrawLine(Pens.Black, 5 + (picChessboard.Width - 14) * i / network.size, 5, 5 + (picChessboard.Width - 14) * i / network.size, picChessboard.Height - 9);
			}
		}

		private void NQueenProblem_Resize(object sender, System.EventArgs e) {
			int size = Math.Min(Size.Height - 80, (Size.Width - 32) / 2);
			size -= ((size + 9) % 13);
			lstValues.Width = size;
			lstValues.Height = size;
			foreach (ColumnHeader c in lstValues.Columns)
				c.Width = (lstValues.Width - 10) / network.size;
			lstValues.Refresh();
			picChessboard.Left = Size.Width / 2;  // place in direct center of form
			picChessboard.Width = size;
			picChessboard.Height = size;
			picChessboard.Refresh();
		}

		private void NQueenProblem_Closing(object sender, System.ComponentModel.CancelEventArgs e) {
			if (solvethread != null) {
				solvethread.Abort();  // send abort signal
				solvethread = null;
			}
		}

		private void timerUpdate_Elapsed(object sender, System.Timers.ElapsedEventArgs e) {
			if (redraw == true) {
				SetNetworkList();
				picChessboard.Refresh();
				txtStatus.Refresh();
				redraw = false;
			}
		}

		private void btnReset_Click(object sender, System.EventArgs e) {
			network.Reset();
			SetNetworkList();
			picChessboard.Refresh();
			txtStatus.Text = "Status: Ready.";
			txtStatus.Refresh();
		}

		#endregion

		#region Helper functions

		public void EnableButtons(bool enabled) {
			nudSize.Enabled = btnSolve.Enabled = enabled;
		}

		private void SetNetworkList() {
			// This function fills the Network listbox with neural network status information.
			lstValues.BeginUpdate();
			lstValues.Columns.Clear();
			ColumnHeader ch = new ColumnHeader();
			for (int i = 0; i < network.size; i++) {
				ColumnHeader c = new ColumnHeader();
				c.Width = (lstValues.Width - 10) / network.size;
				lstValues.Columns.Add(c);
			}
			lstValues.Items.Clear();
			ListViewItem row;
			for (int i = 0; i < network.size; i++) {
				row = new ListViewItem(Math.Round(network.V[i,0], 2).ToString());
				for (int j = 1; j < network.size; j++)
					row.SubItems.Add(Math.Round(network.V[i,j], 2).ToString());
				lstValues.Items.Add(row);
			}
			row = new ListViewItem("");
			lstValues.Items.Add(row);
			for (int i = 0; i < network.size; i++) {
				row = new ListViewItem(Math.Round(network.u[i,0], 2).ToString());
				for (int j = 1; j < network.size; j++)
					row.SubItems.Add(Math.Round(network.u[i,j], 2).ToString());
				lstValues.Items.Add(row);
			}
			lstValues.EndUpdate();
			lstValues.Refresh();
		}

		private void Solve() {
			// This function forms the core of the thread for solving the neural network.
			do {
				network.Analyze();
				txtStatus.Text = "Status: Running: E=" + network.E;
				redraw = true;
			} while (network.E != 0.0);
			btnSolveFull.Text = "Solve (&Full)";
			btnSolveFull.Refresh();
			EnableButtons(true);
			txtStatus.Text = "Status: Solved. E=" + network.E;
			txtStatus.Refresh();
			solvethread = null;
		}

		#endregion

	}
}
